@@ -14,8 +14,9 @@ GNU Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.
 """
+# ruff: noqa: SLF001
 
-__version__ = "0.6.0"
+__version__ = "0.7.0"
 
 import abc
 import contextlib
@@ -23,22 +24,24 @@ import dataclasses
 import datetime
 import sys
 import time
-import traceback
+import traceback as tracebacklib
 import uuid
 import warnings
 from enum import IntEnum
-from types import TracebackType
+from types import TracebackType  # noqa: TCH003
 from typing import (
     Any,
+    ClassVar,
+    Dict,
     List,
     Literal,
-    NoReturn,
     Optional,
     Protocol,
     Tuple,
     Type,
     TypeVar,
     Union,
+    overload,
     runtime_checkable,
 )
 
@@ -51,7 +54,15 @@ with contextlib.suppress(Exception):
 
 UnionType = type(Union[int, str])
 T = TypeVar("T")
-DEFAULT_FORMAT = "[{lvl} {time} line: {line}] {indent}{msg}"  # noqa: FS003
+DEFAULT_FORMAT = (
+    "[{name} {lvl} {time} line: {line}] {indent}{msg}"  # noqa: FS003
+)
+# dataclass' kw_only argument was introduced in python 3.10
+# i don't want to make the requirement 3.10 from 3.8, so instead we check
+# whether or not kw_only is a thing. if it is we'll use it, if it's not, we'll
+# ignore it
+DATACLASS_HAS_KW_ONLY = "kw_only" in dataclasses.dataclass.__kwdefaults__
+DATACLASS_KW_ONLY = {"kw_only": True} if DATACLASS_HAS_KW_ONLY else {}
 
 
 class Level(IntEnum):
@@ -74,14 +85,29 @@ DEFAULT_THRESHOLD = Level.warning
 class Stringable(Protocol):
     """Protocol for stringable objects."""
 
-    def __str__(self) -> str:
+    def __str__(self) -> str:  # pragma: no cover
         """Return the string representation of the object."""
+        return ""
 
 
 Levelable = Union[Level, Stringable, int]
 
 
-def to_level(lvl: Levelable, int_ok: bool = False) -> Union[Level, int]:
+@overload
+def to_level(
+    lvl: Levelable, int_ok: Literal[True] = False
+) -> Union[Level, int]:  # pragma: no cover
+    ...
+
+
+@overload
+def to_level(
+    lvl: Levelable, int_ok: Literal[False] = False
+) -> Level:  # pragma: no cover
+    ...
+
+
+def to_level(lvl: Levelable, int_ok=False):
     """
     Convert a Levelable to a Level (or int).
 
@@ -116,115 +142,13 @@ def to_level(lvl: Levelable, int_ok: bool = False) -> Union[Level, int]:
 NoneType = type(None)
 
 
-def _check_types_error(
-    arg: str, expected: Union[type, Tuple[type, ...]], got: Any
-) -> NoReturn:
-    """
-    Raise a TypeError with the correct message.
-
-    Args:
-        arg (str): The argument name.
-        expected (Union[type, Tuple[type, ...]]): The expected type.
-        got (Any): The got value.
-
-    Raises:
-        TypeError
-
-    Returns:
-        NoReturn: Never returns; raises.
-    """
-    raise TypeError(
-        f"{arg!r} must be type {expected!r}, got {type(got)!r} ({got!r})"
-    )
-
-
-def is_union(union: Any) -> bool:
-    """
-    Is `union` a Union?
-
-    >>> is_union(Union[int, str])
-    True
-
-    Args:
-        union (Any): The object to check.
-
-    Returns:
-        bool: True if `union` is a Union, False otherwise.
-    """
-    try:
-        return (
-            (type(union) is Union)
-            or (type(union) is UnionType)
-            or (union.__origin__ is Union)  # type: ignore
-        )
-    except AttributeError:
-        return False
-
-
-#    Actually: Union   vvv
-#                      ~~~
-def check_union(union: Any, got: Any) -> bool:
-    try:
-        return isinstance(got, union)
-    except TypeError:  # pragma: no cover
-        return any(isinstance(got, typ) for typ in union.__args__)
-
-
-def check_types(
-    **kwargs: Tuple[Union[type, Tuple[type, ...]], Any]
-) -> Literal[True]:
-    """
-    Check types.
-
-    >>> check_types(a=(int, 1), b=(str, "2"))
-    True
-    >>> check_types(arg_1=((int, float), 1), b_8=(str, "Hello world"))
-    True
-    >>> check_types(do=(bool, True))
-    True
-    >>> check_types(do=(bool, None))  # !
-    Traceback (most recent call last):
-    TypeError: 'do' must be type <class 'bool'>, got <class 'NoneType'> (None)
-    >>> check_types(do=(bool, False), sure=((bool, int), 6.9))  # !
-    Traceback (most recent call last):
-    TypeError: 'sure' must be type (<class 'bool'>, <class 'int'>), got\
- <class 'float'> (6.9)
-
-    Args:
-        **kwargs (Tuple[Union[type, Tuple[type, ...]], Any]): The args,\
- expected types and got values.
-
-    Raises:
-        TypeError: If the types are not correct.
-
-    Returns:
-        Literal[True]: Always True.
-    """
-    for arg, (expected, got) in kwargs.items():
-        # ! This union thing is needed, because you cannot use isinstance with
-        # ! unions in python 3.9 and below.
-        if is_union(expected) and check_union(expected, got):
-            continue
-        if not isinstance(got, expected):
-            _check_types_error(arg, expected, got)
-    return True
-
-
+@dataclasses.dataclass(order=True)
 class SetAttr:
     """A context manager for setting and then resetting an attribute."""
 
-    def __init__(self, obj: object, name: str, new_value: Any) -> None:
-        """
-        Initialize the SetAttr.
-
-        Args:
-            obj (object): The object to set the attribute on.
-            name (str): The name of the attribute.
-            new_value (Any): The new value of the attribute.
-        """
-        self.obj = obj
-        self.name = name
-        self.new_value = new_value
+    obj: object
+    name: str
+    new_value: Any
 
     def __enter__(self) -> "SetAttr":
         """
@@ -237,20 +161,17 @@ class SetAttr:
         setattr(self.obj, self.name, self.new_value)
         return self
 
-    def __exit__(self, *args):
+    def __exit__(
+        self,
+        typ: Type[BaseException],
+        value: BaseException,
+        tb: TracebackType,
+    ) -> None:
         """Exit the context manager."""
         setattr(self.obj, self.name, self.old_value)
 
-    def __repr__(self) -> str:  # pragma: no cover
-        return (
-            f"{self.__class__.__module__}.{self.__class__.__qualname__}"
-            f"(obj={self.obj!r},"
-            f" name={self.name!r},"
-            f" new_value={self.new_value!r})"
-        )
 
-
-@dataclasses.dataclass
+@dataclasses.dataclass(order=True, frozen=True)
 class LogEvent:
     """A log event."""
 
@@ -266,7 +187,7 @@ class LogEvent:
 class StreamProtocol(Protocol):
     """Protocol for streams."""
 
-    def write(self, __s: str, /) -> int:
+    def write(self, __s: str, /) -> int:  # pragma: no cover
         """
         Writes `__s` to the stream.
 
@@ -276,6 +197,7 @@ class StreamProtocol(Protocol):
         Returns:
             int: `len(__s)`
         """
+        return 0
 
     def flush(self) -> None:
         """Flushes the streams."""
@@ -295,6 +217,7 @@ class Handler(abc.ABC):
         """
 
 
+@dataclasses.dataclass
 class NoHandler(Handler):
     """A handler that does nothing."""
 
@@ -307,41 +230,15 @@ class NoHandler(Handler):
             event (LogEvent): The event.
         """
 
-    def __repr__(self) -> str:  # pragma: no cover
-        return f"{self.__class__.__module__}.{self.__class__.__qualname__}()"
-
 
+@dataclasses.dataclass(order=True, frozen=True)
 class StreamWriterHandler(Handler):
     """A handler to write to a stream."""
 
-    def __init__(
-        self,
-        stream: StreamProtocol,
-        *,
-        flush: bool = True,
-        use_colors: bool = True,
-        format_msg: bool = True,
-    ) -> None:
-        """
-        Initialize the StreamWriterHandler.
-
-        Args:
-            stream (StreamProtocol): The stream to write to.
-            flush (bool, optional): Flush the stream after writing to it?\
- Defaults to True.
-            use_colors (bool, optional): Use colors? Defaults to True.
-            format_msg (bool, optional): Format the message? Defaults to True.
-        """
-        check_types(
-            stream=(StreamProtocol, stream),
-            flush=(bool, flush),
-            use_colors=(bool, use_colors),
-            format_msg=(bool, format_msg),
-        )
-        self.stream = stream
-        self.flush = flush
-        self.use_colors = use_colors
-        self.format_msg = format_msg
+    stream: StreamProtocol
+    flush: bool = True
+    use_colors: bool = True
+    format_msg: bool = True
 
     def handle(self, logger: "Logger", event: LogEvent) -> None:
         """
@@ -361,29 +258,60 @@ class StreamWriterHandler(Handler):
         if self.flush:
             self.stream.flush()
 
-    def __repr__(self) -> str:  # pragma: no cover
-        return (
-            f"{self.__class__.__module__}.{self.__class__.__qualname__}"
-            f"(stream={self.stream!r},"
-            f" flush={self.flush!r},"
-            f" use_colors={self.use_colors!r},"
-            f" format_msg={self.format_msg!r})"
-        )
+
+@dataclasses.dataclass(order=True, frozen=True, **DATACLASS_KW_ONLY)
+class AttributesToInherit:
+    """
+    What attributes to inherit.
+    Changes made to the parent logger won't be made to the children.
+    """
+
+    propagate: bool = False
+    _id: bool = False
+    list: bool = False  # noqa: A003
+    indent: bool = False
+    enabled: bool = False
+    ctxmgr: bool = False
+    # IndentLogger (used by ctxmgr) takes the logger as an argument.
+    # If it's inherited the PARENT LOGGER will be indented
+    format: bool = True  # noqa: A003
+    threshold: bool = True
+    handlers: bool = True
 
 
 class Logger:
     """THE logger class. You shouldn't really call __init__; use the root\
  logger, or the root logger's get_child"""
 
-    colors: bool = True
+    colors: ClassVar[bool] = True
     # ↪ Should level_to_str use colors? Should be set manually.
+    compare_using_name: ClassVar[bool] = False
+    # ↪ Compare (__eq__, __ne__) using self.name? If False, use self._id
+    attributes_to_inherit: ClassVar[
+        AttributesToInherit
+    ] = AttributesToInherit()
+    # ↪ What attributes should get inherited in ._inherit()
+    color_config: ClassVar[Dict[str, Tuple[Any, ...]]] = {
+        "DEBUG": ("blue",),
+        "INFO": ("cyan",),
+        "WARNING": ("yellow",),
+        "ERROR": ("red",),
+        "CRITICAL": ("red", "on_yellow", ["bold", "underline", "blink"]),
+    }
+    level_name_width: ClassVar[int] = 8
+
+    @classmethod
+    def _thing_to_compare(cls, obj: "Logger") -> str:
+        if cls.compare_using_name:
+            return obj.name
+        return obj._id
 
     def __eq__(self, __o: object) -> bool:
         # sourcery skip: assign-if-exp, reintroduce-else
         if isinstance(
             __o, Logger  # Hardcoded, because class can be subclassed
         ):
-            return self._id == __o._id
+            return self._thing_to_compare(self) == self._thing_to_compare(__o)
         return NotImplemented
 
     def __ne__(self, __o: object) -> bool:
@@ -391,32 +319,56 @@ class Logger:
         if isinstance(
             __o, Logger  # Hardcoded, because class can be subclassed
         ):
-            return self._id != __o._id
+            return self._thing_to_compare(self) != self._thing_to_compare(__o)
+
         return NotImplemented
 
     def __repr__(self) -> str:  # pragma: no cover
-        return (
-            f"<Logger {self.__class__.__module__}."
-            f"{self.__class__.__qualname__} with id {self._id}>"
-        )
+        return f"<{self.__class__.__qualname__} {self.name}>"
 
-    def _inherit(self) -> None:
-        # Made a separate function so it can be overwritten
+    def _inherit(self) -> None:  # noqa: PLR0912,C901  # pragma: no cover
+        if self.higher is None:
+            raise ValueError("Cannot inherit if higher is None.")
 
-        # These (in my opinion) should not be inherited.
-        self.propagate = False
-        self._id = uuid.uuid4()  # Only used for ==, !=
-        self.list: List[LogEvent] = []
-        self.indent = 0  # Should be set manually
-        self.enabled: bool = True
-        self.ctxmgr = IndentLogger(self)
+        if self.attributes_to_inherit.propagate:
+            self.propagate = self.higher.propagate
+        else:
+            self.propagate = False
+        if self.attributes_to_inherit._id:
+            self._id = self.higher._id
+        else:
+            self._id = uuid.uuid4()
+        if self.attributes_to_inherit.list:
+            self.list = self.higher.list
+        else:
+            self.list: List[LogEvent] = []
+        if self.attributes_to_inherit.indent:
+            self.indent = self.higher.indent
+        else:
+            self.indent = 0
+        if self.attributes_to_inherit.enabled:
+            self.enabled = self.higher.enabled
+        else:
+            self.enabled: bool = True
+        if self.attributes_to_inherit.ctxmgr:
+            self.ctxmgr = self.higher.ctxmgr
+        else:
+            self.ctxmgr = IndentLogger(self)
+        if self.attributes_to_inherit.format:
+            self.format: str = self.higher.format
+        else:
+            self.format = DEFAULT_FORMAT
+        if self.attributes_to_inherit.threshold:
+            self.threshold: Union[Level, int] = self.higher.threshold
+        else:
+            self.threshold = DEFAULT_THRESHOLD
+        if self.attributes_to_inherit.handlers:
+            self.handlers: List[Handler] = self.higher.handlers
+        else:
+            self.handlers = self.get_default_handlers()
 
-        # These (in my opinion) should be inherited.
-        self.format: str = self.higher.format
-        self.threshold: Level = self.higher.threshold
-        self.handlers: List[Handler] = self.higher.handlers
-
-    def get_default_handlers(self) -> List[Handler]:
+    @staticmethod
+    def get_default_handlers() -> List[Handler]:
         """
         Get the default handlers. Please note, that overwriting this function\
  will not change the handlers.
@@ -426,11 +378,12 @@ class Logger:
         """
         return [StreamWriterHandler(sys.stderr)]
 
-    def __init__(self, higher: Optional["Logger"] = None) -> None:
+    def __init__(self, name: str, higher: Optional["Logger"] = None) -> None:
         """
         Initialize the logger.
 
         Args:
+            name (str): The name of the logger.
             higher (Optional[Logger], optional): The higher logger. If it's\
  None or omitted then the logger will be the root. Defaults to None.
 
@@ -438,13 +391,13 @@ class Logger:
             ValueError: If the higher logger is None, but root is already\
  created (internally)
         """
-        check_types(higher=((Logger, NoneType), higher))
         if (higher is None) and (not _allow_root):
             raise ValueError(
                 "Cannot create a new logger: Root logger already"
                 " exists. Use that, or make a child logger from the root"
                 " one."
             )
+        self.name = name
         if higher is not None:
             self.higher = higher
             # Made a separate function so it can be overwritten
@@ -455,15 +408,15 @@ class Logger:
         self.list: List[LogEvent] = []
         self.indent = 0  # Should be set manually
         self.format: str = DEFAULT_FORMAT
-        self.threshold: Level = DEFAULT_THRESHOLD
+        self.threshold: Union[Level, int] = DEFAULT_THRESHOLD
         self.enabled: bool = True
         self._id = uuid.uuid4()  # Only used for ==, !=
         self.handlers: List[Handler] = self.get_default_handlers()
 
         self.ctxmgr = IndentLogger(self)
 
-    @staticmethod
-    def _color(rv: str) -> str:
+    @classmethod
+    def color(cls, rv: str) -> str:
         """
         Colorize the string. This function is only recommended internally.
 
@@ -473,25 +426,14 @@ class Logger:
         Returns:
             str: The colorized string.
         """
-        check_types(rv=(str, rv))
-        if rv == "DEBUG":
-            rv = termcolor.colored("DEBUG".ljust(8), "blue")
-        elif rv == "INFO":
-            rv = termcolor.colored("INFO".ljust(8), "cyan")
-        elif rv == "WARNING":
-            rv = termcolor.colored("WARNING".ljust(8), "yellow")
-        elif rv == "ERROR":
-            rv = termcolor.colored("ERROR".ljust(8), "red")
-        elif rv == "CRITICAL":
-            rv = termcolor.colored(
-                "CRITICAL".ljust(8),
-                "red",
-                "on_yellow",
-                ["bold", "underline", "blink"],
+        if rv in cls.color_config:
+            return termcolor.colored(
+                rv.ljust(cls.level_name_width), *cls.color_config[rv]
             )
-        return rv  # noqa: R504
+        return rv
 
-    def level_to_str(self, lvl: Levelable) -> str:
+    @classmethod
+    def level_to_str(cls, lvl: Levelable) -> str:
         """
         Convert a level to a string.
 
@@ -501,19 +443,18 @@ class Logger:
         Returns:
             str: The string.
         """
-        check_types(lvl=(Levelable, lvl))
         try:
             rv = to_level(lvl, False).name.upper()  # type: ignore
         except (ValueError, AttributeError):
-            rv = str(lvl).ljust(8)
+            rv = str(lvl).ljust(cls.level_name_width)
 
         if rv == "WARN":  # pragma: no cover
             rv = "WARNING"
         if rv == "FATAL":  # pragma: no cover
             rv = "CRITICAL"
 
-        if self.colors:
-            rv = self._color(rv)
+        if cls.colors:
+            rv = cls.color(rv)
 
         return rv  # noqa: R504
 
@@ -527,19 +468,21 @@ class Logger:
         Returns:
             str: The formatted message.
         """
-        check_types(event=(LogEvent, event))
         _indent = "  " * self.indent
         _lvl = self.level_to_str(event.level)
-        _time = str(datetime.datetime.fromtimestamp(event.time))
+        _time = str(
+            datetime.datetime.fromtimestamp(event.time)  # noqa: DTZ006
+        )
         _line = str(sys._getframe(event.frame_depth).f_lineno).zfill(5)
         _msg = str(event.msg)
+        _name = str(self.name)
         if (event.tb) and (sys.exc_info() == (None, None, None)):
             warnings.warn(
                 "No traceback available, but tb=True",
                 UserWarning,
-                event.frame_depth - 1,
+                stacklevel=event.frame_depth - 1,
             )
-        _tb = ("\n" + traceback.format_exc()) if event.tb else ""
+        _tb = ("\n" + tracebacklib.format_exc()) if event.tb else ""
         return (
             self.format.format(
                 indent=_indent,
@@ -547,6 +490,7 @@ class Logger:
                 time=_time,
                 line=_line,
                 msg=_msg,
+                name=_name,
             )
             + _tb
             + "\n"
@@ -573,12 +517,6 @@ class Logger:
         Returns:
             LogEvent: The log event.
         """
-        check_types(
-            lvl=(Levelable, lvl),
-            msg=(Stringable, msg),
-            frame_depth=(int, frame_depth),
-            tb=(bool, tb),
-        )
         event = LogEvent(
             msg=str(msg),
             level=to_level(lvl, True),
@@ -614,12 +552,6 @@ class Logger:
         Returns:
             Optional[LogEvent]: The log event if created.
         """
-        check_types(
-            lvl=(Levelable, lvl),
-            msg=(Stringable, msg),
-            traceback=(bool, traceback),
-            frame_depth=(int, frame_depth),
-        )
         # Check if disabled
         if not self.enabled:
             return None
@@ -656,7 +588,6 @@ class Logger:
         Returns:
             int: The number of characters written.
         """
-        check_types(msg=(Stringable, msg), traceback=(bool, traceback))
         return self._log(Level.debug, msg, traceback, 5)
 
     def info(
@@ -673,7 +604,6 @@ class Logger:
         Returns:
             int: The number of characters written.
         """
-        check_types(msg=(Stringable, msg), traceback=(bool, traceback))
         return self._log(Level.info, msg, traceback, 5)
 
     def warning(
@@ -690,7 +620,6 @@ class Logger:
         Returns:
             int: The number of characters written.
         """
-        check_types(msg=(Stringable, msg), traceback=(bool, traceback))
         return self._log(Level.warning, msg, traceback, 5)
 
     def error(
@@ -707,7 +636,6 @@ class Logger:
         Returns:
             int: The number of characters written.
         """
-        check_types(msg=(Stringable, msg), traceback=(bool, traceback))
         return self._log(Level.error, msg, traceback, 5)
 
     def critical(
@@ -724,10 +652,9 @@ class Logger:
         Returns:
             int: The number of characters written.
         """
-        check_types(msg=(Stringable, msg), traceback=(bool, traceback))
         return self._log(Level.critical, msg, traceback, 5)
 
-    def get_child(self) -> "Logger":
+    def get_child(self, name: str) -> "Logger":
         """
         Get a child logger.
 
@@ -737,7 +664,7 @@ class Logger:
         # This function should be short, so if the user doesn't like it, it
         # can be copy-pasted, and the user can change it.
         # (That's why we have `._inherit`)
-        return type(self)(self)
+        return type(self)(name, self)
 
     def is_enabled_for(self, lvl: Levelable) -> bool:
         """
@@ -749,7 +676,6 @@ class Logger:
         Returns:
             bool: Whether the logger is enabled for the given level.
         """
-        check_types(lvl=(Levelable, lvl))
         lvl = to_level(lvl, True)
         if not isinstance(self.threshold, Level):
             warnings.warn(
@@ -757,23 +683,17 @@ class Logger:
                 f" {type(self.threshold)!r} ({self.threshold!r})."
                 " Converting threshold...",
                 UserWarning,
+                stacklevel=2,
             )
             self.threshold = to_level(self.threshold, True)
         return lvl >= self.threshold
 
 
+@dataclasses.dataclass(order=True)
 class IndentLogger:
     """Indent the logger."""
 
-    def __init__(self, logger: Logger) -> None:
-        """
-        Initialize the indent logger.
-
-        Args:
-            logger (Logger): The logger.
-        """
-        check_types(logger=(Logger, logger))
-        self.logger = logger
+    logger: Logger
 
     def __enter__(self) -> int:
         """
@@ -801,13 +721,8 @@ class IndentLogger:
         """
         self.logger.indent -= 1
 
-    def __repr__(self) -> str:  # pragma: no cover
-        return (
-            f"{self.__class__.__module__}.{self.__class__.__qualname__}"
-            f"(logger={self.logger!r})"
-        )
-
 
+@dataclasses.dataclass(order=True)
 class ChangeThreshold:
     """Change the threshold for a logger."""
 
@@ -819,16 +734,15 @@ class ChangeThreshold:
             logger (Logger): The logger.
             level (Levelable): The new threshold.
         """
-        check_types(logger=(Logger, logger), level=(Levelable, level))
         self.logger = logger
         self.level = to_level(level, True)
 
-    def __enter__(self) -> Level:
+    def __enter__(self) -> Union[Level, int]:
         """
         Change the threshold.
 
         Returns:
-            Level: The old threshold.
+            Union[Level, int]: The old threshold.
         """
         self.old_level = self.logger.threshold
         self.logger.threshold = self.level  # type: ignore
@@ -850,14 +764,7 @@ class ChangeThreshold:
         """
         self.logger.threshold = self.old_level
 
-    def __repr__(self) -> str:  # pragma: no cover
-        return (
-            f"{self.__class__.__module__}.{self.__class__.__qualname__}"
-            f"(logger={self.logger!r},"
-            f" level={self.level!r})"
-        )
-
 
 _allow_root = True
-root = Logger()
+root = Logger("root")
 _allow_root = False
